@@ -248,7 +248,7 @@ namespace FSpot {
 				size = size_spin.ValueAsInt;
 
 			command_thread = new System.Threading.Thread (new System.Threading.ThreadStart (Upload));
-			command_thread.Name = Mono.Posix.Catalog.GetString ("Transfering Pictures");
+			command_thread.Name = Mono.Posix.Catalog.GetString ("Transferring Pictures");
 
 			//FIXME: get the files/dirs count in a cleaner way than (* 5 + 2(zip) + 9)
 			// selection * 5 (original, mq, lq, thumbs, comments)
@@ -668,6 +668,7 @@ namespace FSpot {
 			writer.RenderBeginTag ("div");
 
 			if (i > 0)
+				// Abbreviation of previous	
 				WritePageNav (writer, "prev", PhotoIndexPath (i - 1), Mono.Posix.Catalog.GetString("Prev"));
 
 			WritePageNav (writer, "index", IndexPath (i / perpage), Mono.Posix.Catalog.GetString("Index"));
@@ -781,7 +782,7 @@ namespace FSpot {
 			writer.AddAttribute ("class", "footer");
 			writer.RenderBeginTag ("div");
 			
-			writer.Write (Mono.Posix.Catalog.GetString ("Gallery generated by "));
+			writer.Write (Mono.Posix.Catalog.GetString ("Gallery generated by") + " ");
 			
 			writer.AddAttribute ("href", "http://www.gnome.org/projects/f-spot");
 			writer.RenderBeginTag ("a");
