@@ -5,6 +5,8 @@ namespace FSpot {
 		private Gtk.ScrolledWindow scroll;
 		private Delay hide;
 
+		public ScrolledView (System.IntPtr raw) : base (raw) {}
+
 		public ScrolledView () : base () {
 			scroll = new Gtk.ScrolledWindow  (null, null);
 			this.Put (scroll, 0, 0);
@@ -15,6 +17,7 @@ namespace FSpot {
 			ebox.ShowAll ();
 			
 			hide = new Delay (2000, new GLib.IdleHandler (HideControls));
+			this.Destroyed += HandleDestroyed;
 		}
 
 		public bool HideControls ()
@@ -31,9 +34,8 @@ namespace FSpot {
 			ebox.Show ();
 		}
 
-		protected override void OnDestroyed ()
+		private void HandleDestroyed (object sender, System.EventArgs args)
 		{
-			base.OnDestroyed ();
 			hide.Stop ();
 		}
 
@@ -61,13 +63,13 @@ namespace FSpot {
 		private Gtk.Button forward_button;
 		private Gtk.Button back_button;
 		
-		public FullScreenView (PhotoQuery query) : base ("Full Screen Mode")
+		public FullScreenView (IBrowsableCollection collection) : base ("Full Screen Mode")
 		{
 			try {
 				//scroll = new Gtk.ScrolledWindow (null, null);
 				
 				scroll = new ScrolledView ();
-				view = new PhotoImageView (query);
+				view = new PhotoImageView (collection);
 				view.ModifyBg (Gtk.StateType.Normal, this.Style.Black);
 				view.PointerMode = ImageView.PointerModeType.Scroll;
 				this.Add (scroll);
