@@ -19,7 +19,7 @@ namespace FSpot {
 		static ImageFile ()
 		{
 			name_table = new System.Collections.Hashtable ();
-			name_table [".svg"] = typeof (ImageFile);
+			name_table [".svg"] = typeof (FSpot.Svg.SvgFile);
 			name_table [".gif"] = typeof (ImageFile);
 			name_table [".jpeg"] = typeof (JpegFile);
 			name_table [".jpg"] = typeof (JpegFile);
@@ -44,12 +44,18 @@ namespace FSpot {
 			}
 		}
 
-
 		public PixbufOrientation Orientation {
 			get {
 				return GetOrientation ();
 			}
 		}
+
+		public virtual string Description
+		{
+			get { 
+				return null;
+			}
+		}
 		
 		public virtual void Save (Gdk.Pixbuf pixbuf, System.IO.Stream stream)
 		{
@@ -85,11 +91,13 @@ namespace FSpot {
 			return PixbufOrientation.TopLeft;
 		}
 		
-		public virtual System.DateTime Date () 
+		public virtual System.DateTime Date 
 		{
-			return File.GetCreationTimeUtc  (this.path);
+			get {
+				return File.GetCreationTimeUtc  (this.path);
+			}
 		}
-		
+
 		public static bool HasLoader (string path)
 		{
 			string extension = System.IO.Path.GetExtension (path).ToLower ();
