@@ -310,6 +310,9 @@ public class MainWindow {
 
 	public void SetViewMode (ModeType value)
 	{
+		if (view_mode == value)
+			return;
+
 		view_mode = value;
 		switch (view_mode) {
 		case ModeType.IconView:
@@ -447,7 +450,7 @@ public class MainWindow {
 				if (icon == null) {
 					// FIXME this needs a lot more work.
 					try {
-						Pixbuf tmp = PixbufUtils.LoadAtMaxSize (query.Photos[num].DefaultVersionPath, 128, 128);
+						Pixbuf tmp = FSpot.PhotoLoader.LoadAtMaxSize (query.Items [num], 128, 128);
 						icon = PixbufUtils.TagIconFromPixbuf (tmp);
 						tmp.Dispose ();
 					} catch {
@@ -1013,7 +1016,7 @@ public class MainWindow {
 		}
 
 		info_display = new FSpot.InfoDisplay ();
-		info_display_window = new Gtk.Dialog ("EXIF Data", main_window, 
+		info_display_window = new Gtk.Dialog ("Metadata Browser", main_window, 
 						      Gtk.DialogFlags.NoSeparator | Gtk.DialogFlags.DestroyWithParent);
 		info_display_window.SetDefaultSize (400, 400);
 		Gtk.ScrolledWindow scroll = new ScrolledWindow ();
@@ -1147,10 +1150,7 @@ public class MainWindow {
 
 	void HandleCloseCommand (object sender, EventArgs args)
 	{
-		// FIXME
-		// Should use Application.Quit(), but for that to work we need to terminate the threads
-		// first too.
-		Environment.Exit (0);
+		this.Window.Destroy ();
 	}
 	
 	void HandleCreateVersionCommand (object obj, EventArgs args)
@@ -1365,9 +1365,10 @@ public class MainWindow {
 			info_vpaned.Show ();
 	}
 
-	public void Present ()
-	{
-		main_window.Present ();
+	public Gtk.Window Window {
+		get {
+			return main_window;
+		}
 	}
 	
 	void HandleViewSlideShow (object sender, EventArgs args)
@@ -1580,18 +1581,43 @@ public class MainWindow {
 	public void HandleDeleteSelectedTagCommand (object sender, EventArgs args)
 	{
 		Tag [] tags = this.tag_selection_widget.TagHighlight ();
- 		string header = Mono.Posix.Catalog.GetPluralString ("Delete the selected tag?",
-								 "Delete the {0} selected tags?", 
-								 tags.Length);
+		bool can_delete = true;
 
+
+		System.Array.Sort (tags, new TagRemoveComparer ());
+		string header = Mono.Posix.Catalog.GetPluralString ("Delete the selected tag?",
+								    "Delete the {0} selected tags?", 
+								    tags.Length);
+		
 		header = String.Format (header, tags.Length);
 		string msg = Mono.Posix.Catalog.GetString("If you delete a tag, all associations with photos are lost.");
 		string ok_caption = Mono.Posix.Catalog.GetPluralString ("_Delete tag", "_Delete tags", tags.Length);
-		 
- 		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, DialogFlags.DestroyWithParent, MessageType.Warning, header, msg, ok_caption)) {                              
- 			db.Photos.Remove (tags);
-			icon_view.QueueDraw ();
- 		}
+		
+		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, 
+									   DialogFlags.DestroyWithParent, 
+									   MessageType.Warning, 
+									   header, 
+									   msg, 
+									   ok_caption)) {                              
+			try { 				
+				db.Photos.Remove (tags);
+			} catch (InvalidTagOperationException e) {
+				System.Console.WriteLine ("this is something or another");
+
+				// A Category is not empty. Can not delete it.
+				string error_msg = Mono.Posix.Catalog.GetString ("Category is not empty");
+				string error_desc = String.Format (Mono.Posix.Catalog.GetString ("Can not delete categorys which has tags. Please delete tags under \"{0}\" first"),
+						     e.Tag.Name);
+				
+				HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
+									    Gtk.MessageType.Error, ButtonsType.Ok, 
+									    error_msg,
+									    error_desc);
+				md.Run ();
+				md.Destroy ();
+			}
+		}
+		icon_view.QueueDraw ();
 	}
 
 	void HandleUpdateThumbnailCommand (object sende, EventArgs args)
@@ -1707,10 +1733,10 @@ public class MainWindow {
 
 	void HandleQueryItemChanged (FSpot.IBrowsableCollection browsable, int item)
 	{
-		if (info_box.Photo == (browsable [item] as Photo))
+		if (photo_view.Item.Index == item) {
+			UpdateMenus ();
 			info_box.Update ();
-
-		UpdateMenus ();
+		}
 	}
 	//
 	// Handle Main Menu 
