@@ -22,8 +22,14 @@ namespace FSpot {
 			this.item = new BrowsablePointer (query, -1);
 			item.IndexChanged += PhotoIndexChanged;
 			this.Destroyed += HandleDestroyed;
+			this.SetTransparentColor (this.Style.BaseColors [(int)Gtk.StateType.Normal]);
 		}
 		
+		protected override void OnStyleSet (Gtk.Style previous)
+		{
+			this.SetTransparentColor (this.Style.Backgrounds [(int)Gtk.StateType.Normal]);
+		}
+
 		new public BrowsablePointer Item {
 			get {
 				return item;
@@ -135,16 +141,31 @@ namespace FSpot {
 				// FIXME in some cases the image passes completely through the
 				// pixbuf loader without properly loading... I'm not sure what to do about this other
 				// than try to load the image one last time.
-				this.Pixbuf = FSpot.PhotoLoader.Load (item.Collection, 
-								      item.Index);
+				this.Pixbuf = null;
+				if (!loader.Loading) {
+					try {
+						System.Console.WriteLine ("Falling back to file loader");
+
+						this.Pixbuf = FSpot.PhotoLoader.Load (item.Collection, 
+										      item.Index);
+					} catch (System.Exception e) {
+						if (!(e is GLib.GException))
+							System.Console.WriteLine (e.ToString ());
+					}
+				}
+
 				if (this.Pixbuf == null)
 					this.Pixbuf = new Gdk.Pixbuf (PixbufUtils.ErrorPixbuf, 0, 0, 
 								      PixbufUtils.ErrorPixbuf.Width, 
 								      PixbufUtils.ErrorPixbuf.Height);
+
+				this.ZoomFit ();
 			} else {
 				this.Pixbuf = loader.Pixbuf;
+
+				if (!loader.Prepared)
+					this.ZoomFit ();
 			}
-			this.ZoomFit ();
 
 			if (prev != this.Pixbuf && prev != null)
 				prev.Dispose ();
