@@ -532,23 +532,18 @@ public class PhotoStore : DbStore {
 	public Photo Create (string path, out Pixbuf thumbnail)
 	{
 		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
-		DateTime time = img.Date ();
-		return Create (time, path, out thumbnail);
-	}
-
-	public Photo Create (DateTime time_in_utc, string path, out Pixbuf thumbnail)
-	{
-		uint unix_time = DbUtils.UnixTimeFromDateTime (time_in_utc);
-
+		uint unix_time = DbUtils.UnixTimeFromDateTime (img.Date);
+		string description = img.Description != null ? img.Description : "";
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 
 		command.CommandText = String.Format ("INSERT INTO photos (time, " +
 						     "directory_path, name, description, default_version_id) " +
-						     "       VALUES ({0}, '{1}', '{2}', '', {3})                                       ",
+						     "       VALUES ({0}, '{1}', '{2}', '{3}', {4})                                       ",
 						     unix_time,
 						     SqlString (System.IO.Path.GetDirectoryName (path)),
 						     SqlString (System.IO.Path.GetFileName (path)),
+						     SqlString (description),
 						     Photo.OriginalVersionId);
 
 		command.ExecuteScalar ();
@@ -1062,11 +1057,12 @@ public class PhotoStore : DbStore {
 		bool hide = true;
 		if (tags != null) {
 			foreach (Tag t in tags) {
-				if (t.Id == tag_store.Hidden.Id) 
+				if (tag_store.Hidden == null || t.Id == tag_store.Hidden.Id) 
 					hide = false;
 			}
-		}
-		
+		} else
+			hide = tag_store.Hidden != null;
+
 		// The SQL query that we want to construct is:
 		//
 		// SELECT photos.id
@@ -1103,23 +1099,26 @@ public class PhotoStore : DbStore {
 		}
 		
 		if (tags != null && tags.Length > 0) {
-				bool first = true;
-				foreach (Tag t in tags) {
-					if (t.Id == tag_store.Hidden.Id)
-						continue;
-					
-					if (first) {
-						query_builder.Append (String.Format ("{0} photos.id IN (SELECT photo_id FROM photo_tags WHERE tag_id IN (",
-										     hide || range != null ? " AND " : " WHERE "));
-					}
-					
-					query_builder.Append (String.Format ("{0}{1} ", first ? "" : ", ", t.Id));
-					
-					first = false;
+			bool first = true;
+			foreach (Tag t in tags) {
+				if (t == null)
+					continue;
+				
+				if (tag_store.Hidden != null && t.Id == tag_store.Hidden.Id)
+					continue;
+				
+				if (first) {
+					query_builder.Append (String.Format ("{0} photos.id IN (SELECT photo_id FROM photo_tags WHERE tag_id IN (",
+									     hide || range != null ? " AND " : " WHERE "));
 				}
 				
-				if (!first)
-					query_builder.Append (")) ");
+				query_builder.Append (String.Format ("{0}{1} ", first ? "" : ", ", t.Id));
+				
+					first = false;
+			}
+			
+			if (!first)
+				query_builder.Append (")) ");
 		}
 		
 		query_builder.Append ("ORDER BY photos.time");
