@@ -175,6 +175,11 @@ public class PhotoView : EventBox {
 			description_entry.Sensitive = false;
 			description_entry.Text = "";
 		}
+		if (description_entry.Text == "")
+			tips.SetTip (description_entry, description_entry.Text, "This is a tip");
+		else
+			tips.SetTip (description_entry, null, "This is a tip");
+
 		description_entry.Changed += HandleDescriptionChanged;
 	}    
 
@@ -227,9 +232,20 @@ public class PhotoView : EventBox {
 	private void ProcessImage (bool redeye)
 	{
 		int x, y, width, height;
-		if (! photo_view.GetSelection (out x, out y, out width, out height))
+		if (! photo_view.GetSelection (out x, out y, out width, out height)) {
+			string msg = Mono.Posix.Catalog.GetString ("No selection available");
+			string desc = Mono.Posix.Catalog.GetString ("This tool requires an active selection. Please select a region of the image and try the operation again");
+			
+			HigMessageDialog md = new HigMessageDialog ((Gtk.Window)this.Toplevel, DialogFlags.DestroyWithParent, 
+								    Gtk.MessageType.Error, ButtonsType.Ok, 
+								    msg,
+								    desc);
+
+			md.Run ();
+			md.Destroy ();
 			return;
-		
+		}		
+
 		Pixbuf original_pixbuf = photo_view.CompletePixbuf ();
 		if (original_pixbuf == null) {
 			return;
@@ -312,7 +328,7 @@ public class PhotoView : EventBox {
 			FSpot.ImageFile img = FSpot.ImageFile.Create (photo.DefaultVersionPath);
 			if (img is FSpot.JpegFile) {
 				FSpot.JpegFile jimg = img as FSpot.JpegFile;
-				jimg.Description = photo.Description;
+				jimg.SetDescription (photo.Description);
 				jimg.SaveMetaData (photo.DefaultVersionPath);
 			}
 			//Query.Store.Commit (photo);
@@ -336,6 +352,7 @@ public class PhotoView : EventBox {
 			else
 				CommitPendingChanges ();
 
+		tips.SetTip (description_entry, description_entry.Text, "This is a tip");
 		description_photo = Item.Index;
 		description_delay.Start ();
 	}
@@ -473,6 +490,10 @@ public class PhotoView : EventBox {
 		tips.SetTip (crop_button, Mono.Posix.Catalog.GetString ("Crop image to selected area"), "so is this one");
 		tips.SetTip (redeye_button, Mono.Posix.Catalog.GetString ("Remove redeye from selected area"), "this is one too");
 		tips.SetTip (color_button, Mono.Posix.Catalog.GetString ("Adjust the image colors"), "This is a tip");
+		tips.SetTip (constraints_option_menu, Mono.Posix.Catalog.GetString ("Constrain the aspect ratio of the selection"), "This is a tip");
+		tips.SetTip (display_next_button, Mono.Posix.Catalog.GetString ("Next image"), "This is a tip");
+		tips.SetTip (display_previous_button, Mono.Posix.Catalog.GetString ("Previous image"), "This is a tip");
+
 	}
 
 }
