@@ -1,19 +1,17 @@
 using ICSharpCode.SharpZipLib.Zip.Compression;
+using SemWeb;
 
 namespace FSpot.Png {
-	public class PngFile : ImageFile {
+	public class PngFile : ImageFile, SemWeb.StatementSource {
 		System.Collections.ArrayList chunk_list;
 		
 		public PngFile (string path) : base (path)
 		{
 			this.path = path;
-			using (System.IO.Stream input = System.IO.File.OpenRead (this.Path)) {
-				Load (input);
-			}
 		}
 
 		/**
-		   Title 	Short (one line) title or caption for image
+		   Title 	Short (one line) title or caption for image 
 		   Author 	Name of image's creator
 		   Description 	Description of image (possibly long)
 		   Copyright 	Copyright notice
@@ -24,14 +22,107 @@ namespace FSpot.Png {
 		   Source 	Device used to create the image
 		   Comment 	Miscellaneous comment
 		   
-		   xmp is iTXt:XML:com.adobe.xmp
+		   xmp is XML:com.adobe.xmp
 
 		   Other keywords may be defined for other purposes. Keywords of general interest can be registered with th
 		*/
-		
+		public void Select (SemWeb.StatementSink sink)
+		{
+			foreach (Chunk c in Chunks) {
+				if (c is IhdrChunk) {
+					IhdrChunk ih = c as IhdrChunk;
+					MetadataStore.AddLiteral (sink, "tiff:ImageWidth", ih.Width.ToString ());
+					MetadataStore.AddLiteral (sink, "tiff:ImageLength", ih.Height.ToString ());
+				} else if(c is TimeChunk) {
+					TimeChunk tc = c as TimeChunk;
+
+					MetadataStore.AddLiteral (sink, "xmp:ModifyDate", tc.Time.ToString ("yyyy-MM-ddThh:mm:ss"));
+				} else if (c is TextChunk) {
+					TextChunk text = c as TextChunk;
+
+					switch (text.Keyword) {
+					case "XMP":
+					case "XML:com.adobe.xmp":
+						System.IO.Stream xmpstream = new System.IO.MemoryStream (text.TextData);
+						FSpot.Xmp.XmpFile xmp = new FSpot.Xmp.XmpFile (xmpstream);
+						xmp.Select (sink);
+						break;
+					case "Comment":
+						MetadataStore.AddLiteral (sink, "exif:UserComment", text.Text);
+						break;
+					case "Software":
+						break;
+					case "Title":
+						MetadataStore.AddLiteral (sink, "dc:title", "rdf:Alt", new Literal (text.Text, "x-default", null));
+						break;
+					case "Author":
+						MetadataStore.AddLiteral (sink, "dc:creator", "rdf:Seq", new Literal (text.Text));
+						break;
+					case "Copyright":
+						MetadataStore.AddLiteral (sink, "dc:rights", "rdf:Alt", new Literal (text.Text, "x-default", null));
+						break;
+					case "Description":
+						MetadataStore.AddLiteral (sink, "dc:description", "rdf:Alt", new Literal (text.Text, "x-default", null));
+						break;
+					case "Creation Time":
+						try {
+							System.DateTime time = System.DateTime.Parse (text.Text);
+							MetadataStore.AddLiteral (sink, "xmp:CreateDate", time.ToString ("yyyy-MM-ddThh:mm:ss"));
+						} catch (System.Exception e) {
+							System.Console.WriteLine (e.ToString ());
+						}
+						break;
+					}
+				} else if (c is ColorChunk) {
+					ColorChunk color = (ColorChunk)c;
+					string [] whitepoint = new string [2];
+					whitepoint [0] = color.WhiteX.ToString ();
+					whitepoint [1] = color.WhiteY.ToString ();
+					MetadataStore.Add (sink, "tiff:WhitePoint", "rdf:Seq", whitepoint);
+					int i = 0;
+					string [] rgb = new string [6];
+					rgb [i++] = color.RedX.ToString ();
+					rgb [i++] = color.RedY.ToString ();
+					rgb [i++] = color.GreenX.ToString ();
+					rgb [i++] = color.GreenY.ToString ();
+					rgb [i++] = color.BlueX.ToString ();
+					rgb [i++] = color.BlueY.ToString ();
+					MetadataStore.Add (sink, "tiff:PrimaryChromaticities", "rdf:Seq", rgb);
+				} else if (c.Name == "sRGB") {
+					MetadataStore.AddLiteral (sink, "exif:ColorSpace", "1");
+				} else if (c is PhysChunk) {
+					PhysChunk phys = (PhysChunk)c;
+					uint denominator = (uint) (phys.InMeters ? 100 : 1);
+					
+					MetadataStore.AddLiteral (sink, "tiff:ResolutionUnit", phys.InMeters ? "3" : "1");
+					MetadataStore.AddLiteral (sink, "tiff:XResolution", new FSpot.Tiff.Rational (phys.PixelsPerUnitX, denominator).ToString ());
+					MetadataStore.AddLiteral (sink, "tiff:YResolution", new FSpot.Tiff.Rational (phys.PixelsPerUnitY, denominator).ToString ());
+				}
+			}
+		}
+
+		public System.Collections.ArrayList Chunks {
+			get {
+				if (chunk_list == null) {
+					using (System.IO.Stream input = System.IO.File.OpenRead (this.Path)) {
+						Load (input);
+					}
+				}
+				
+				return chunk_list;
+			}
+		}
+
 		public class ZtxtChunk : TextChunk {
 			//public static string Name = "zTXt";
 
+			protected bool compressed = true;
+			public bool Compressed {
+				get {
+					return compressed;
+				}
+			}
+			
 			byte compression;
 			public byte Compression {
 			        get {
@@ -52,16 +143,40 @@ namespace FSpot.Png {
 				i++;
 				Compression = data [i++];
 
-				byte [] inflated = Chunk.Inflate (data, i, data.Length - i);
-				text = TextChunk.Latin1.GetString (inflated, 0, inflated.Length);
+				text_data = Chunk.Inflate (data, i, data.Length - i);
 			}
 		}
 
+		public class PhysChunk : Chunk {
+			public PhysChunk (string name, byte [] data) : base (name, data) {}
+			
+			public uint PixelsPerUnitX {
+				get {
+					return BitConverter.ToUInt32 (data, 0, false);
+				}
+			}
+
+			public uint PixelsPerUnitY {
+				get {
+					return BitConverter.ToUInt32 (data, 4, false);
+				}
+			}
+			
+			public bool InMeters {
+				get {
+					return data [8] == 0;
+				}
+			}
+		}
+		
 		public class TextChunk : Chunk {
 			//public static string Name = "tEXt";
 
 			protected string keyword;
 			protected string text;
+			protected byte [] text_data;
+			protected System.Text.Encoding encoding = Latin1;
+
 			public static System.Text.Encoding Latin1 = System.Text.Encoding.GetEncoding (28591);
 			public TextChunk (string name, byte [] data) : base (name, data) {}
 
@@ -71,7 +186,9 @@ namespace FSpot.Png {
 
 				keyword = GetString (ref i);
 				i++;
-				text = TextChunk.Latin1.GetString (data, i, data.Length - i);
+				int len = data.Length - i;
+				text_data = new byte [len];
+				System.Array.Copy (data, i, text_data, 0, len);
 			}
 
 			public string Keyword {
@@ -80,9 +197,16 @@ namespace FSpot.Png {
 				}
 			}
 
+			public byte [] TextData 
+			{
+				get {
+					return text_data;
+				}
+			}
+			
 			public string Text {
 				get {
-					return text;
+					return encoding.GetString (text_data, 0, text_data.Length);
 				}
 			}
 		}
@@ -122,26 +246,33 @@ namespace FSpot.Png {
 			//public static string Name = "zTXt";
 
 			string Language;
-			bool Compressed;
+			string LocalizedKeyword;
 
 			public override void Load (byte [] data)
 			{
 				int i = 0;
 				keyword = GetString (ref i);
 				i++;
-				Compressed = (data [i++] != 0);
+				compressed = (data [i++] != 0);
 				Compression = data [i++];
 				Language = GetString (ref i);
 				i++;
+				LocalizedKeyword = GetString (ref i, System.Text.Encoding.UTF8);
+				i++;
+
 				if (Compressed) {
-					byte [] inflated = Chunk.Inflate (data, i, data.Length - i);
-					text = TextChunk.Latin1.GetString (inflated, 0, inflated.Length);
+					text_data = Chunk.Inflate (data, i, data.Length - i);
 				} else {
-					text = TextChunk.Latin1.GetString (data, i, data.Length - i);
+					int len = data.Length - i;
+					text_data = new byte [len];
+					System.Array.Copy (data, i, text_data, 0, len);
 				}
 			}
 
-			public ItxtChunk (string name, byte [] data) : base (name, data) {}
+			public ItxtChunk (string name, byte [] data) : base (name, data) 
+			{
+				encoding = System.Text.Encoding.UTF8;
+			}
 		}
 
 		public class TimeChunk : Chunk {
@@ -170,6 +301,54 @@ namespace FSpot.Png {
 			public TimeChunk (string name, byte [] data) : base (name, data) {}
 		}
 
+		public class ColorChunk : Chunk {
+			// FIXME this should be represented like a tiff rational
+			public const uint Denominator = 100000;
+
+			public ColorChunk (string name, byte [] data) : base (name, data) {}
+
+			public FSpot.Tiff.Rational WhiteX {
+				get {
+					return new FSpot.Tiff.Rational (FSpot.BitConverter.ToUInt32 (data, 0, false), Denominator);
+				}
+			}
+			public FSpot.Tiff.Rational WhiteY {
+				get { 
+					return new FSpot.Tiff.Rational (FSpot.BitConverter.ToUInt32 (data, 4, false), Denominator);
+				}
+			}
+			public FSpot.Tiff.Rational RedX {
+				get { 
+					return new FSpot.Tiff.Rational (FSpot.BitConverter.ToUInt32 (data, 8, false), Denominator);
+				}
+			}
+			public FSpot.Tiff.Rational RedY {
+				get { 
+					return new FSpot.Tiff.Rational (FSpot.BitConverter.ToUInt32 (data, 12, false), Denominator);
+				}
+			}
+			public FSpot.Tiff.Rational GreenX {
+				get { 
+					return new FSpot.Tiff.Rational (FSpot.BitConverter.ToUInt32 (data, 16, false), Denominator);
+				}
+			}
+			public FSpot.Tiff.Rational GreenY {
+				get { 
+					return new FSpot.Tiff.Rational (FSpot.BitConverter.ToUInt32 (data, 20, false), Denominator);
+				}
+			}
+			public FSpot.Tiff.Rational BlueX {
+				get { 
+					return new FSpot.Tiff.Rational (FSpot.BitConverter.ToUInt32 (data, 24, false), Denominator);
+				}
+			}
+			public FSpot.Tiff.Rational BlueY {
+				get { 
+					return new FSpot.Tiff.Rational (FSpot.BitConverter.ToUInt32 (data, 28, false), Denominator);
+				}
+			}
+		}
+
 		public enum ColorType : byte {
 			Gray = 0,
 			Rgb = 2,
@@ -288,12 +467,14 @@ namespace FSpot.Png {
 
 				name_table = new System.Collections.Hashtable ();
 				name_table ["iTXt"] = typeof (ItxtChunk);
-				name_table ["tXMP"] = typeof (TextChunk);
+				name_table ["tXMP"] = typeof (ItxtChunk);
 				name_table ["tEXt"] = typeof (TextChunk);
 				name_table ["zTXt"] = typeof (ZtxtChunk);
 				name_table ["tIME"] = typeof (TimeChunk);
 				name_table ["iCCP"] = typeof (IccpChunk);
 				name_table ["IHDR"] = typeof (IhdrChunk);
+				name_table ["cHRM"] = typeof (ColorChunk);
+				name_table ["pHYs"] = typeof (PhysChunk);
 			}
 			
 			public Chunk (string name, byte [] data) 
@@ -303,14 +484,20 @@ namespace FSpot.Png {
 				Load (data);
 			}
 
-			protected string GetString  (ref int i) 
+			
+			protected string GetString  (ref int i, System.Text.Encoding enc) 
 			{
 				for (; i < data.Length; i++) {
 					if (data [i] == 0)
 						break;
 				}	
 				
-				return TextChunk.Latin1.GetString (data, 0, i);
+				return enc.GetString (data, 0, i);
+			}
+
+			protected string GetString  (ref int i) 
+			{
+				return GetString (ref i, TextChunk.Latin1);
 			}
 
 			public virtual void Load (byte [] data)
@@ -652,7 +839,7 @@ namespace FSpot.Png {
 			Chunk palette = null;
 			Chunk transparent = null;
 
-			foreach (Chunk chunk in chunk_list) {
+			foreach (Chunk chunk in Chunks) {
 				if (chunk.Name == "IDAT")
 					ci.Add (chunk);
 				else if (chunk.Name == "PLTE") 
@@ -661,7 +848,7 @@ namespace FSpot.Png {
 					transparent = chunk;
 			}
 
-			IhdrChunk ihdr = (IhdrChunk) chunk_list [0];
+			IhdrChunk ihdr = (IhdrChunk) Chunks [0];
 			System.Console.WriteLine ("Attempting to to inflate image {0}.{1}({2}, {3})", ihdr.Color, ihdr.Depth, ihdr.Width, ihdr.Height);
 			ScanlineDecoder decoder = new ScanlineDecoder (ci, ihdr.GetScanlineLength (0), ihdr.Height);
 			decoder.Fill ();
@@ -753,7 +940,7 @@ namespace FSpot.Png {
 			    heading [5] != 10 ||
 			    heading [6] != 26 ||
 			    heading [7] != 10)
-			    throw new System.Exception ("This ain't no png file");
+			    throw new System.Exception ("Invalid PNG magic number");
 
 			chunk_list = new System.Collections.ArrayList ();
 
@@ -774,20 +961,17 @@ namespace FSpot.Png {
 				//System.Console.Write ("read one {0} {1}", chunk, chunk.Name);
 				chunk_list.Add (chunk);
 
-				/*
+#if TEST_METADATA				
 				if (chunk is TextChunk) {
 					TextChunk text = (TextChunk) chunk;
 					System.Console.Write (" Text Chunk {0} {1}", 
-							      text.Keyword, text.Text);
+							      text.Keyword, "", text.Text);
 				}
-				*/
 
-				/*
 				TimeChunk time = chunk as TimeChunk;
 				if (time != null)
 					System.Console.Write(" Time {0}", time.Time);
-				*/
-
+#endif
 				//System.Console.WriteLine ("");
 				
 				if (chunk.Name == "IEND")
@@ -795,6 +979,51 @@ namespace FSpot.Png {
 			}
 		}
 
+		public string LookupText (string keyword)
+		{
+			TextChunk chunk = LookupTextChunk (keyword);
+			if (chunk != null)
+				return chunk.Text;
+
+			return null;
+		}
+
+		public TextChunk LookupTextChunk (string keyword)
+		{
+			foreach (Chunk chunk in Chunks) {
+				TextChunk text = chunk as TextChunk;
+				if (text != null && text.Keyword == keyword)
+					return text;
+			}
+			return null;	
+		}
+
+
+		public override string Description {
+			get {
+				string description = LookupText ("Description");
+
+				if (description != null)
+					return description;
+				else
+					return LookupText ("Comment");
+			}
+		}
+
+		public override System.DateTime Date {
+			get {
+				// FIXME: we should first try parsing the
+				// LookupText ("Creation Time") as a valid date
+
+				foreach (Chunk chunk in Chunks) {
+					TimeChunk time = chunk as TimeChunk;
+					if (time != null)
+						return time.Time.ToUniversalTime ();
+				}
+				return base.Date;
+			}
+		}
+
 #if false
 		public class ImageFile {
 			string Path;
