@@ -365,6 +365,9 @@ public class TagCommands {
 			Gdk.Pixbuf tmp = null;
 		       
 			image_view.GetSelection (out x, out y, out width, out height);
+			if (width > 0 && height > 0) 
+				icon_view.Selection.Clear ();
+				
 			if (image_view.Pixbuf != null) {
 				if (width > 0 && height > 0) {
 					tmp = new Gdk.Pixbuf (image_view.Pixbuf, x, y, width, height);
@@ -406,7 +409,7 @@ public class TagCommands {
 			this.CreateDialog ("edit_icon_dialog");
 
 			if (t is Category) {
-				this.Dialog.Title = String.Format (Mono.Posix.Catalog.GetString ("Edit icon For category {0}"), t.Name);
+				this.Dialog.Title = String.Format (Mono.Posix.Catalog.GetString ("Edit icon for category {0}"), t.Name);
 			} else {
 				this.Dialog.Title = String.Format (Mono.Posix.Catalog.GetString ("Edit icon for tag {0}"), t.Name);
 			}
@@ -414,7 +417,11 @@ public class TagCommands {
 			preview_image.Pixbuf = t.Icon;
 
 			query = new FSpot.PhotoQuery (db.Photos);
-			query.Tags = new Tag [] { t, db.Tags.Hidden };
+			
+			if (db.Tags.Hidden != null)
+				query.Tags = new Tag [] { t, db.Tags.Hidden };
+			else 
+				query.Tags = new Tag [] { t };
 
 			image_view = new FSpot.PhotoImageView (query);
 			image_view.SelectionXyRatio = 1.0;
