@@ -18,12 +18,7 @@ public class PixbufSerializer {
 	{
 		Pixdata pixdata = new Pixdata ();
 
-		IntPtr error = IntPtr.Zero;
-		bool raw_ret = gdk_pixdata_deserialize (ref pixdata, (uint) data.Length, data, out error);
-		bool ret = raw_ret;
-
-		if (error != IntPtr.Zero)
-			throw new GLib.GException (error);
+		pixdata.Deserialize ((uint) data.Length, data);
 
 		return Pixbuf.FromPixdata (pixdata, true);
 	}
@@ -31,6 +26,9 @@ public class PixbufSerializer {
 	[DllImport("libgdk_pixbuf-2.0-0.dll")]
 	static extern IntPtr gdk_pixdata_serialize(ref Gdk.Pixdata raw, out uint stream_length_p);
 
+	[DllImport("libglib-2.0-0.dll")]
+	static extern void g_free(IntPtr ptr);
+
 	public static byte [] Serialize (Pixbuf pixbuf)
 	{
 		Pixdata pixdata = new Pixdata ();
@@ -41,6 +39,8 @@ public class PixbufSerializer {
 
 		byte [] data = new byte [data_length];
 		Marshal.Copy (raw_data, data, 0, (int) data_length);
+		
+		g_free (raw_data);
 
 		return data;
 	}
@@ -173,9 +173,64 @@ public class Category : Tag {
 	}
 }
 
+public class InvalidTagOperationException : InvalidOperationException {
+	public Tag tag;
+	
+	public InvalidTagOperationException (Tag t, string message) : base (message)
+	{
+		tag = t;
+	}
+
+	public Tag Tag {
+		get {
+			return tag;
+		}
+	}
+
+}
+
+public class TagRemoveComparer : IComparer {
+	public int Compare (object obj1, object obj2) 
+	{
+		Tag t1 = obj1 as Tag;
+		Tag t2 = obj2 as Tag;
+		
+		return Compare (t1, t2);
+	}
+       
+	public int Compare (Tag t1, Tag t2)
+	{
+		bool c1 = t1 is Category;
+		bool c2 = t2 is Category;
+		
+		if (c1 == c2) {
+			if (c1 && c2) {
+				for (Category cat = t1 as Category; cat.Category != null; cat = cat.Category) {
+					if (cat == t2.Category)
+						return 1;
+				}
+			}
+			
+			return t1.Name.CompareTo (t2.Name);
+		}
+
+		if (c1)
+			return 1;
+		else
+			return -1;
+	}
+}
+
+public delegate void TagCreatedHandler(Tag t);
+public delegate void TagChangedHandler(Tag t);
+public delegate void TagDeletedHandler(Tag t);
 
 public class TagStore : DbStore {
 
+	public event TagCreatedHandler TagCreated;
+	public event TagChangedHandler TagChanged;
+	public event TagDeletedHandler TagDeleted;
+	
 	Category root_category;
 	public Category RootCategory {
 		get {
@@ -195,6 +250,13 @@ public class TagStore : DbStore {
 			tag.Icon = PixbufSerializer.Deserialize (Convert.FromBase64String (icon_string));
 	}
 
+	private Tag hidden;
+	public Tag Hidden {
+		get {
+			return hidden;
+		}
+	}
+
 	// In this store we keep all the items (i.e. the tags) in memory at all times.  This is
 	// mostly to simplify handling of the parent relationship between tags, but it also makes it
 	// a little bit faster.  We achieve this by passing "true" as the cache_is_immortal to our
@@ -225,7 +287,11 @@ public class TagStore : DbStore {
 
 			tag.SortPriority = Convert.ToInt32 (reader[3]);
 			AddToCache (tag);
+			
+			if (tag.Name == "Hidden")
+				hidden = tag;
 		}
+
 		reader.Close ();
 		command.Dispose ();
 
@@ -285,6 +351,7 @@ public class TagStore : DbStore {
 		Tag hidden_tag = CreateTag (RootCategory, "Hidden");
 		hidden_tag.StockIconName = "f-spot-hidden.png";
 		hidden_tag.SortPriority = -9;
+		this.hidden = hidden_tag;
 		Commit (hidden_tag);
 
 		Tag people_category = CreateCategory (RootCategory, "People");
@@ -298,12 +365,12 @@ public class TagStore : DbStore {
 		Commit (places_category);
 
 		Tag events_category = CreateCategory (RootCategory, "Events");
-		places_category.StockIconName = "f-spot-events.png";
+		events_category.StockIconName = "f-spot-events.png";
 		events_category.SortPriority = -7;
 		Commit (events_category);
 
 		Tag other_category = CreateCategory (RootCategory, "Other");
-		places_category.StockIconName = "f-spot-other.png";
+		other_category.StockIconName = "f-spot-other.png";
 		other_category.SortPriority = -6;
 		Commit (other_category);
 	}
@@ -353,7 +420,8 @@ public class TagStore : DbStore {
 
 		Tag tag = new Tag (category, id, name);
 		AddToCache (tag);
-
+		if (TagCreated != null)
+			TagCreated(tag);	
 		return tag;
 	}
 
@@ -366,7 +434,8 @@ public class TagStore : DbStore {
 
 		Category new_category = new Category (parent_category, id, name);
 		AddToCache (new_category);
-
+		if (TagCreated != null)
+			TagCreated(new_category);
 		return new_category;
 	}
 
@@ -377,21 +446,19 @@ public class TagStore : DbStore {
 		else
 			return LookupInCache (id) as Tag;
 	}
-
+	
 	public override void Remove (DbItem item)
 	{
+		Category category = item as Category;
+		if (category != null && 
+		    category.Children != null && 
+		    category.Children.Length > 0)
+			throw new InvalidTagOperationException (category, "Cannot remove category that contains children");
+
 		RemoveFromCache (item);
 		
 		((Tag)item).Category = null;
 		
-		// FIXME this should either throw an exception or make one
-		// command, not recurse.
-		if (item is Category) {
-			Category category = (Category)item;
-			foreach (Tag tag in category.Children)
-				Remove (tag);
-		}
-
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 
@@ -399,6 +466,7 @@ public class TagStore : DbStore {
 		command.ExecuteNonQuery ();
 
 		command.Dispose ();
+		TagDeleted((Tag)item);
 	}
 
 
@@ -436,6 +504,9 @@ public class TagStore : DbStore {
 		command.ExecuteNonQuery ();
 
 		command.Dispose ();
+		
+		if (TagChanged != null)
+			TagChanged(tag);
 	}
 
 
