@@ -1,4 +1,5 @@
 using FSpot;
+using SemWeb;
 
 namespace FSpot.Tiff {
 
@@ -6,21 +7,21 @@ namespace FSpot.Tiff {
 	// because they differ from the tiff standard names
 	public enum NiffId : ushort {
 		SubfileType                     = 0x00fe,
-			PelPathLength                   = 0x0100,
-			LineProgressionLength           = 257,
-			BitsPerSample                   = 0x0101,
-			PhotometricInterpretation       = 0x0106,
-			DataOffset                      = 0x0111,
-			SamplesPerPixel 		= 0x0115,
-			DataByteCounts                  = 0x0117,
-			PelPathResolution               = 0x011a,
-			LineProgressionResolution       = 0x011b,
-			ResolutionUnit  		= 0x0128,
-			ColumnsPerPelPath               = 322,
-			RowsPerLineProgression          = 323,
-			Rotation                        = 33465,
-			NavyCompression                 = 33466,
-			TileIndex                       = 33467
+		PelPathLength                   = 0x0100,
+		LineProgressionLength           = 257,
+		BitsPerSample                   = 0x0101,
+		PhotometricInterpretation       = 0x0106,
+		DataOffset                      = 0x0111,
+		SamplesPerPixel 		= 0x0115,
+		DataByteCounts                  = 0x0117,
+		PelPathResolution               = 0x011a,
+		LineProgressionResolution       = 0x011b,
+		ResolutionUnit  		= 0x0128,
+		ColumnsPerPelPath               = 322,
+		RowsPerLineProgression          = 323,
+	        Rotation                        = 33465,
+	        NavyCompression                 = 33466,
+		TileIndex                       = 33467
 	}
 
 	public enum TagId : ushort {
@@ -96,9 +97,9 @@ namespace FSpot.Tiff {
 		JPEGDCTables                    = 0x0208,
 		JPEGACTables                    = 0x0209,
 
-		YCBCRCoefficients		= 0x0211,
-		YCBCRSubSampling		= 0x0212,
-		YCBCRPositioning		= 0x0213,
+		YCbCrCoefficients		= 0x0211,
+		YCbCrSubSampling		= 0x0212,
+		YCbCrPositioning		= 0x0213,
 
 		ReferenceBlackWhite		= 0x0214,
 		RelatedImageFileFormat   	= 0x1000,
@@ -253,6 +254,115 @@ namespace FSpot.Tiff {
 		PimIfdPointer                   = 0xc4a5
 	}
 
+	public struct SRational {
+		public int Numerator;
+		public int Denominator;
+		
+		public SRational (int numerator, int denominator)
+		{
+			Numerator = numerator;
+			Denominator = denominator;
+		}
+
+		public static SRational BitwiseCopy (Rational rational)
+		{
+			SRational result;
+
+			result.Numerator = unchecked ((int) rational.Numerator);
+			result.Denominator = unchecked ((int) rational.Denominator);
+			return result;
+		}
+
+		public override string ToString ()
+		{
+			if (Denominator == 1)
+				return Numerator.ToString ();
+
+			return System.String.Format ("{0}/{1}", Numerator, Denominator);
+		}
+		
+		public double Value {
+			get {
+				return Numerator / (double)Denominator;
+			}
+		}
+	}
+
+	public struct Rational {
+		public uint Numerator;
+		public uint Denominator;
+
+		public Rational (uint numerator, uint denominator)
+		{
+			Numerator = numerator;
+			Denominator = denominator;
+		}
+
+		public override string ToString ()
+		{
+			if (Denominator == 1)
+				return Numerator.ToString ();
+			
+			return System.String.Format ("{0}/{1}", Numerator, Denominator);
+		}
+		
+		public double Value {
+			get {
+				return Numerator / (double)Denominator;
+			}
+		}
+	}
+
+	struct UserComment {
+		string Charset;
+		public string Value;
+
+		public UserComment (byte [] raw_data, bool little)
+		{
+			string charset = System.Text.Encoding.ASCII.GetString (raw_data, 0, 8);
+			string value = null;
+			System.Text.Encoding enc;
+
+			switch (charset) {
+			case "ASCII\0\0\0":
+				enc = System.Text.Encoding.ASCII;
+				break;
+			case "UNICODE\0":
+				enc = System.Text.Encoding.Unicode;
+				break;
+			case "SJIS\0\0\0\0":
+				// FIXME I'm pretty sure this isn't actually the encoding name.
+				enc = System.Text.Encoding.GetEncoding ("SJIS");
+				break;
+			case "\0\0\0\0\0\0\0\0":
+				// FIXME the spec says to use the local encoding in this case, we could probably
+				// do something smarter, but whatever.
+				enc = System.Text.Encoding.Default;
+				break;
+			default:
+				enc = null;
+				throw new System.Exception (System.String.Format ("Invalid charset name: {0}", charset));
+			}
+
+			Charset = charset;
+			Value = enc.GetString (raw_data, 8, raw_data.Length - 8);
+		}
+	}
+
+	public struct CFAPattern {
+		public ushort Rows;
+		public ushort Columns;
+		public byte [] Values;
+		
+		public CFAPattern (byte [] raw_data, bool little)
+		{
+			Columns = BitConverter.ToUInt16 (raw_data, 0, little);
+			Rows = BitConverter.ToUInt16 (raw_data, 2, little);
+			Values = new byte [Rows * Columns];
+			System.Array.Copy (raw_data, 4, Values, 0, Values.Length);
+		}
+	}
+
 	public enum ExtraSamples {
 		Unspecified = 0,
 		AssociatedAlpha = 1,
@@ -272,7 +382,7 @@ namespace FSpot.Tiff {
 		ITULab = 10,
 		LogL = 32844, // Log Luminance
 		LogLUV = 32845,
-		CFA = 32803,  // ColorFilterArray... the good stuff
+		ColorFilterArray = 32803,  // ColorFilterArray... the good stuff
 		LinearRaw = 34892  // DBG LinearRaw
 	}
 
@@ -307,6 +417,88 @@ namespace FSpot.Tiff {
 		ReducedResolution = 2,
 		PageOfMultipage = 3
 	}
+	
+	public enum ExposureProgram {
+		NotDefined = 0,
+		Manual = 1,
+		NormalProgram = 2,  // Normal Program
+		AperturePriority = 3, // Aperture priority
+		ShutterPriorty = 4, // Shutter priority
+		CreativeProgram = 5, // Creative program
+		ActionProgram = 6, // Action program
+		PortraitMode = 7, // Portrait mode
+		LandscapeMode = 8 // Landscape mode
+	}
+
+	public enum ExposureMode {
+		Auto = 0,
+		Manual = 1,
+		AutoBracket = 2
+	}
+
+	public enum MeteringMode {
+		Uknown = 0,
+		Average = 1,
+		CenterWeightedAverage = 2,
+		Spot = 3,
+		MulitSpot = 4,
+		Pattern = 5,
+		Partial = 6,
+	}
+
+	public enum LightSource {
+		Unknown = 0,
+		Daylight = 1,
+		Fluorescent = 2,
+		Tungsten = 3,
+		Fash = 4,
+		FineWeather = 9,
+		CloudyWeather = 10,
+		Shade = 11,
+		DaylightFluorescent = 12,
+		DaylightWhiteFluorescent = 13,
+		CoolWhiteFluorescent = 14,
+		WhiteFluorescent = 15,
+		StandardLightA = 17,
+		StandardLightB = 18,
+		StandardLightC = 19,
+		D55 = 20,
+		D65 = 21,
+		D75 = 22,
+		D50 = 23,
+		ISOStudioTungsten = 24,
+		OtherSource = 255
+	}
+
+	public enum ColorSpace : ushort {
+		sRGB = 1,
+		Uncalibrated = 0xffff
+	}
+
+	public enum ComponentsConfiguration {
+		DoesNotExist = 0,
+		Y = 1,
+		Cb = 2,
+		Cr = 3,
+		R = 4,
+		G = 6,
+	}
+
+	public enum ResolutionUnit : ushort {
+	        Uncalibrated = 1,
+		Inch = 2,
+		Centimeter = 3
+	}
+	
+	public enum SensingMethod : short {
+		NotDefined = 1,
+		OneChipColorAreaSensor = 2,
+		TwoChipColorAreaSensor = 3,
+		ThreeChipColorAreaSensor = 4,
+		ColorSequentialAreaSensor = 5,
+		TrilinearSensor = 7,
+		ColorSequentialLinearSensor = 8
+	}
 
 	[System.Flags]
 	public enum NewSubfileType : uint {
@@ -418,7 +610,7 @@ namespace FSpot.Tiff {
 		}
 	}
 
-	public class Header {
+	public class Header : SemWeb.StatementSource {
 		public Endian endian;
 
 		private uint directory_offset;
@@ -426,6 +618,7 @@ namespace FSpot.Tiff {
 
 		public Header (System.IO.Stream stream)
 		{
+			using (new Timer ("new Tiff.Header")) {
 			byte [] data = new byte [8];
 			stream.Read (data, 0, data.Length);
 			if (data [0] == 'M' && data [1] == 'M')
@@ -449,23 +642,174 @@ namespace FSpot.Tiff {
 				break;
 			}
 
-			/*
-			if (data [0] == 'M' && data [1] == 'M' && data [2] == 0 && data [3] == 42)
-				endian = Endian.Big;
-			else if (data [0] == 'I' && data [1] == 'I' && data [2] == 42 && data [3] == 0)
-				endian = Endian.Little;
-			else
-				throw new System.Exception ("Invalid Tiff Header Block");
-			*/
-
-			System.Console.WriteLine ("Converting Something");
+			//System.Console.WriteLine ("Converting Something");
 			directory_offset = BitConverter.ToUInt32 (data, 4, endian == Endian.Little);
 			
 			if (directory_offset < 8)
 				throw new System.Exception ("Invalid IFD0 Offset [" + directory_offset.ToString () + "]"); 
 			
-			System.Console.WriteLine ("Reading First IFD");
+			//System.Console.WriteLine ("Reading First IFD");
 			Directory = new ImageDirectory (stream, directory_offset, endian); 
+			}
+		}
+		
+		
+		public void Select (SemWeb.StatementSink sink)
+		{
+			using (new Timer ("Tiff.Header.Select")) {
+				SelectDirectory (Directory, sink);
+			}
+		}
+
+		private void SelectDirectory (ImageDirectory dir, StatementSink sink)
+		{
+			foreach (DirectoryEntry e in dir.Entries) {
+				//System.Console.WriteLine ("{0}", e.Id);
+				switch (e.Id) {
+				case TagId.IPTCNAA:
+					System.IO.Stream iptcstream = new System.IO.MemoryStream (e.RawData);
+					FSpot.Iptc.IptcFile iptc = new FSpot.Iptc.IptcFile (iptcstream);
+					iptc.Select (sink);
+					break;
+				case TagId.PhotoshopPrivate:
+					System.IO.Stream bimstream = new System.IO.MemoryStream (e.RawData);
+					FSpot.Bim.BimFile bim = new FSpot.Bim.BimFile (bimstream);
+					bim.Select (sink);
+					break;
+				case TagId.XMP:
+					System.IO.Stream xmpstream = new System.IO.MemoryStream (e.RawData);
+					FSpot.Xmp.XmpFile xmp = new FSpot.Xmp.XmpFile (xmpstream);
+					xmp.Select (sink);
+					break;
+				case TagId.ImageDescription:
+					MetadataStore.AddLiteral (sink, "dc:description", "rdf:Alt", 
+								  new Literal (e.ValueAsString [0], "x-default", null));
+					break;
+				case TagId.UserComment:
+					MetadataStore.AddLiteral (sink, "exif:UserComment", "rdf:Alt", 
+								  new Literal (e.ValueAsString [0], "x-default", null));
+					break;
+				case TagId.Copyright:
+					MetadataStore.AddLiteral (sink, "dc:rights", "rdf:Alt", 
+								  new Literal (e.ValueAsString [0], "x-default", null));
+					break;
+				case TagId.Artist:
+					MetadataStore.Add (sink, "dc:creator", "rdf:Seq", e.ValueAsString);
+					break;
+				case TagId.ExifIfdPointer:
+					try {
+						ImageDirectory sub = ((SubdirectoryEntry)e).Directory [0];
+						SelectDirectory (sub, sink);
+					} catch (System.Exception exc) {
+						System.Console.WriteLine (exc);
+					}
+					break;
+				case TagId.Software:
+					MetadataStore.AddLiteral (sink, "xmp:CreatorTool", e.ValueAsString [0]);
+					break;
+				case TagId.DateTime:
+					MetadataStore.AddLiteral (sink, "xmp:ModifyDate", 
+								  e.ValueAsDate.ToString ("yyyy-MM-ddThh:mm:ss"));
+					break;
+				case TagId.DateTimeOriginal:
+				case TagId.DateTimeDigitized:
+					// FIXME subsectime needs to be included in these values
+					// FIXME shouldn't DateTimeOriginal be xmp:CreateDate? the spec says no but wtf?
+					MetadataStore.AddLiteral (sink, "exif:" + e.Id.ToString (), 
+								  e.ValueAsDate.ToString ("yyyy-MM-ddThh:mm:ss"));
+					break;
+					//case TagId.Flash:
+					
+					//case TagId.SpatialFrequencyResponse
+				case TagId.CFAPattern:
+					CFAPattern pattern = new CFAPattern (e.RawData, e.IsLittle);
+					Entity empty = new Entity (null);
+					Statement top = new Statement ("", 
+								       (Entity)MetadataStore.Namespaces.Resolve ("exif:" + e.Id.ToString ()),
+								       empty);
+					
+					Statement cols = new Statement (empty, 
+									(Entity) MetadataStore.Namespaces.Resolve ("exif:Columns"),
+									new Literal (pattern.Columns.ToString (), null, null));
+					sink.Add (cols);
+					Statement rows = new Statement (empty, 
+									(Entity) MetadataStore.Namespaces.Resolve ("exif:Rows"),
+									new Literal (pattern.Rows.ToString (), null, null));
+					sink.Add (rows);
+					string [] vals = e.ArrayToString (pattern.Values);
+					MetadataStore.Add (sink, empty, "exif:Values", "rdf:Seq", vals);
+					sink.Add (top);
+					break;
+				case TagId.ExifVersion:
+				case TagId.FlashPixVersion:
+				case TagId.ColorSpace:
+				case TagId.CompressedBitsPerPixel:
+				case TagId.PixelYDimension:
+				case TagId.PixelXDimension:
+				case TagId.RelatedSoundFile:
+				case TagId.ExposureTime:
+				case TagId.FNumber:
+				case TagId.ExposureProgram:
+				case TagId.SpectralSensitivity:
+				case TagId.ShutterSpeedValue:
+				case TagId.ApertureValue:
+				case TagId.BrightnessValue:
+				case TagId.ExposureBiasValue:
+				case TagId.MaxApertureValue:
+				case TagId.SubjectDistance:
+				case TagId.MeteringMode:
+				case TagId.LightSource:
+				case TagId.FocalLength:
+				case TagId.FlashEnergy:
+				case TagId.FocalPlaneXResolution:
+				case TagId.FocalPlaneYResolution:
+				case TagId.FocalPlaneResolutionUnit:
+				case TagId.ExposureIndex:
+				case TagId.SensingMethod:
+				case TagId.FileSource:
+				case TagId.SceneType:
+				case TagId.CustomRendered:
+				case TagId.ExposureMode:
+				case TagId.WhiteBalance:
+				case TagId.DigitalZoomRatio:
+				case TagId.FocalLengthIn35mmFilm:
+				case TagId.SceneCaptureType:
+				case TagId.GainControl:
+				case TagId.Contrast:
+				case TagId.Saturation:
+				case TagId.Sharpness:
+					MetadataStore.AddLiteral (sink, "exif:" + e.Id.ToString (), e.ValueAsString [0]);
+					break;
+				case TagId.ComponentsConfiguration:
+				case TagId.ISOSpeedRatings:
+				case TagId.SubjectArea:
+				case TagId.SubjectLocation:
+					MetadataStore.Add (sink, "exif:" + e.Id.ToString (), "rdf:Seq", e.ValueAsString);
+					break;
+				case TagId.TransferFunction:
+				case TagId.YCbCrSubSampling:
+				case TagId.WhitePoint:
+				case TagId.PrimaryChromaticities:
+				case TagId.YCbCrCoefficients:
+				case TagId.ReferenceBlackWhite:
+				case TagId.BitsPerSample:
+					MetadataStore.Add (sink, "tiff:" + e.Id.ToString (), "rdf:Seq", e.ValueAsString);
+					break;
+				case TagId.Orientation:
+				case TagId.Compression:
+				case TagId.PhotometricInterpretation:					
+				case TagId.SamplesPerPixel:
+				case TagId.PlanarConfiguration:
+				case TagId.YCbCrPositioning:
+				case TagId.ResolutionUnit:
+				case TagId.ImageWidth:
+				case TagId.ImageLength:
+				case TagId.Model:
+				case TagId.Make:
+					MetadataStore.AddLiteral (sink, "tiff:" + e.Id.ToString (), e.ValueAsString [0]);
+					break;
+				}
+			}
 		}
 
 		public string Dump ()
@@ -515,7 +859,7 @@ namespace FSpot.Tiff {
 		protected virtual void ReadEntries (System.IO.Stream stream) 
 		{
 			num_entries = Converter.ReadUShort (stream, endian);
-			System.Console.WriteLine ("reading {0} entries", num_entries);
+			//System.Console.WriteLine ("reading {0} entries", num_entries);
 
 			entries = new System.Collections.ArrayList (num_entries);
 			int entry_length = num_entries * 12;
@@ -527,7 +871,7 @@ namespace FSpot.Tiff {
 			for (int pos = 0; pos < entry_length; pos += 12) {
 				DirectoryEntry entry = EntryFactory.CreateEntry (this, content, pos, this.endian);
 				entries.Add (entry);		
-				System.Console.WriteLine ("Added Entry {0} {1} - {2} * {3}", entry.Id.ToString (), entry.Id.ToString ("x"), entry.Type, entry.Count);
+				//System.Console.WriteLine ("Added Entry {0} {1} - {2} * {3}", entry.Id.ToString (), entry.Id.ToString ("x"), entry.Type, entry.Count);
 				if (entry.Id == TagId.NewSubfileType) {
 					
 				}
@@ -548,14 +892,14 @@ namespace FSpot.Tiff {
 		
 		protected void LoadNextDirectory (System.IO.Stream stream)
 		{
-			System.Console.WriteLine ("next_directory_offset = {0}", next_directory_offset);
+			//System.Console.WriteLine ("next_directory_offset = {0}", next_directory_offset);
 			next_directory = null;
 			try {
 				if (next_directory_offset != 0)
 					next_directory = new ImageDirectory (stream, next_directory_offset, this.endian);
 
 			} catch (System.Exception e) {
-				System.Console.WriteLine ("Error loading directory {0}", e.ToString ());
+				//System.Console.WriteLine ("Error loading directory {0}", e.ToString ());
 				next_directory = null;
 				next_directory_offset = 0;
 			}		
@@ -579,7 +923,6 @@ namespace FSpot.Tiff {
 				if (entry.Id == id)
 					return entry;
 
-			
 			return null;
 		}
 
@@ -590,17 +933,16 @@ namespace FSpot.Tiff {
 				if ((uint)entry.Id == id)
 					return entry;
 
-			
 			return null;
 		}
 		
 		
 		public void Dump () 
 		{
-			System.Console.WriteLine ("Directory Start");
+			//System.Console.WriteLine ("Directory Start");
 			foreach (DirectoryEntry e in this.Entries)
 				e.Dump ();
-			System.Console.WriteLine ("End Directory");
+			//System.Console.WriteLine ("End Directory");
 		}
 		
 		public string Dump2 ()
@@ -609,9 +951,9 @@ namespace FSpot.Tiff {
 			builder.Append ("Dummping IFD");
 			foreach (DirectoryEntry entry in entries) {
 				builder.Append (entry.ToString ()+ "\n");
-				if (entry is SubdirectoryEntry) {
+
+				if (entry is SubdirectoryEntry)
 					builder.Append ("Found SUBDIRECTORYENTRY\n");
-				}
 			}
 			
 			if (next_directory != null) {
@@ -659,7 +1001,7 @@ namespace FSpot.Tiff {
 			
 			switch (type) {
 			case EntryType.Ifd:
-				System.Console.WriteLine ("Trying to load {0} {1}", tagid, tagid.ToString ("x"));
+				//System.Console.WriteLine ("Trying to load {0} {1}", tagid, tagid.ToString ("x"));
 				return new SubdirectoryEntry (input, start, header_endian);
 			case EntryType.Byte:
 				return new ByteEntry (input, start, header_endian);
@@ -721,20 +1063,20 @@ namespace FSpot.Tiff {
 
 			for (int i = 0; i <  entry_count; i++) {
 				directory_offset = BitConverter.ToUInt32 (raw_data, i * 4, endian == Endian.Little);
-				System.Console.WriteLine ("Entering Subdirectory {0} at {1}", tagid.ToString (), directory_offset);
+				//System.Console.WriteLine ("Entering Subdirectory {0} at {1}", tagid.ToString (), directory_offset);
 				Directory [i] = new ImageDirectory (stream, directory_offset, endian);
-				System.Console.WriteLine ("Leaving Subdirectory {0} at {1}", tagid.ToString (), directory_offset);
+				//System.Console.WriteLine ("Leaving Subdirectory {0} at {1}", tagid.ToString (), directory_offset);
 			}
 		}
 
 		public override void Dump ()
 		{
 			for (int i = 0; i < GetEntryCount (); i++) {
-				 System.Console.WriteLine ("Entering Subdirectory {0}.{2} at {1}", tagid.ToString (), directory_offset, i);
-				 if (Directory [i] != null)
-					 Directory [i].Dump ();
-				 
-				 System.Console.WriteLine ("Leaving Subdirectory {0}.{2} at {1}", tagid.ToString (), directory_offset, i);
+				//System.Console.WriteLine ("Entering Subdirectory {0}.{2} at {1}", tagid.ToString (), directory_offset, i);
+				if (Directory [i] != null)
+					Directory [i].Dump ();
+				
+				//System.Console.WriteLine ("Leaving Subdirectory {0}.{2} at {1}", tagid.ToString (), directory_offset, i);
 			}
 		}
 	}
@@ -743,12 +1085,6 @@ namespace FSpot.Tiff {
 		public ShortEntry (byte [] data, int offset, Endian endian) : base (data, offset, endian)
 		{
 		}
-
-		public new ushort [] Value {
-			get {
-				return this.ShortValue;
-			}
-		}
 	}
 	
 	public class LongEntry : DirectoryEntry {
@@ -757,13 +1093,6 @@ namespace FSpot.Tiff {
 			if (type != EntryType.Long)
 				throw new System.Exception (System.String.Format ("Invalid Settings At Birth {0}", tagid));
 		}
-
-		public new uint [] Value
-		{
-			get {
-				return this.LongValue;
-			}
-		}
 	}
 
 	public class ByteEntry : DirectoryEntry {
@@ -780,10 +1109,8 @@ namespace FSpot.Tiff {
 			if (type != EntryType.Ascii)
 				throw new System.Exception (System.String.Format ("Invalid Settings At Birth {0}", tagid));
 		}
-
-
 	}
-
+	
 #if false
 	public class ImageLoader {
 		ushort width;
@@ -900,6 +1227,12 @@ namespace FSpot.Tiff {
 			}
 		}
 
+		public bool IsLittle {
+			get {
+				return (endian == Endian.Little);
+			}
+		}
+
 		public static int ParseHeader (byte [] data, int start, out TagId tagid, out EntryType type, Endian endian)
 		{
 			tagid = (TagId) BitConverter.ToUInt16 (data, start, endian == Endian.Little);
@@ -925,27 +1258,28 @@ namespace FSpot.Tiff {
 				raw_data = data;
 			}
 
+#if false
 			switch ((int)this.Id) {
 			case (int)TagId.NewSubfileType:
-				System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new NewSubFileType {0}", (NewSubfileType) this.ValueAsLong [0]);
+				//System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new NewSubFileType {0}", (NewSubfileType) this.ValueAsLong [0]);
 				break;
 			case (int)TagId.SubfileType:
-				System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new SubFileType {0}", (SubfileType) this.ValueAsLong [0]);
+				//System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new SubFileType {0}", (SubfileType) this.ValueAsLong [0]);
 				break;
 			case (int)TagId.Compression:
-				System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new Compression {0}", (Compression) this.ValueAsLong [0]);
+				//System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new Compression {0}", (Compression) this.ValueAsLong [0]);
 				
 				break;
 			case (int)TagId.JPEGProc:
-				System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new JPEGProc {0}", (JPEGProc) this.ValueAsLong [0]);
+				//System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new JPEGProc {0}", (JPEGProc) this.ValueAsLong [0]);
 				
 				break;
 			case (int)TagId.PhotometricInterpretation:
-				System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new PhotometricInterpretation {0}", (PhotometricInterpretation) this.ValueAsLong [0]);
+				//System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new PhotometricInterpretation {0}", (PhotometricInterpretation) this.ValueAsLong [0]);
 				break;
 			case (int)TagId.ImageWidth:
 			case (int)TagId.ImageLength:
-				System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new {1} {0}", this.ValueAsLong [0], this.Id);
+				//System.Console.WriteLine ("XXXXXXXXXXXXXXXXXXXXX new {1} {0}", this.ValueAsLong [0], this.Id);
 				break;
 			case 50648:
 			case 50656:
@@ -954,7 +1288,7 @@ namespace FSpot.Tiff {
 				System.Console.WriteLine ("XXXX ", System.Text.Encoding.ASCII.GetString (raw_data));
 				switch (this.type) {
 				case EntryType.Long:
-					foreach (uint val in ((LongEntry)this).LongValue)
+					foreach (uint val in ((LongEntry)this).Value)
 						System.Console.Write (" {0}", val);
 					break;
 				case EntryType.Short:
@@ -969,6 +1303,7 @@ namespace FSpot.Tiff {
 				System.Console.WriteLine ("");
 				break;
 			}
+#endif
 		}
 
 		public virtual void Dump ()
@@ -1014,34 +1349,57 @@ namespace FSpot.Tiff {
 			byte [] tmp = new byte [len + 1];
 			System.Text.Encoding.UTF8.GetBytes (value, 0, value.Length, tmp, 0);
 			tmp[len] = 0;
-			System.Console.WriteLine ("value = {0} len = {1}", value, len);
+			System.Console.WriteLine ("SetData: value = {0} len = {1}", value, len);
 			SetData (tmp);
 		}
 	
-		public static System.DateTime DateTimeFromString(string dt)
-		 {
-			 // Exif DateTime strings are formatted as
-			 //      "YYYY:MM:DD HH:MM:SS"
-
-			 string delimiters = " :";
-			 string[] dt_data = dt.Split ( delimiters.ToCharArray(), 6 );
-			 System.DateTime result;
-			 result = new System.DateTime (System.Int32.Parse(dt_data[0]), 
-						       System.Int32.Parse(dt_data[1]), 
-						       System.Int32.Parse(dt_data[2]),
-						       System.Int32.Parse(dt_data[3]), 
-						       System.Int32.Parse(dt_data[4]), 
-						       System.Int32.Parse(dt_data[5]));
+		public static System.DateTime DateTimeFromString (string dt)
+		{
+			// Exif DateTime strings are formatted as
+			//      "YYYY:MM:DD HH:MM:SS"
+			
+			string delimiters = " :";
+			string[] dt_data = dt.Split ( delimiters.ToCharArray(), 6 );
+			System.DateTime result;
+			result = new System.DateTime (System.Int32.Parse(dt_data[0]), 
+						      System.Int32.Parse(dt_data[1]), 
+						      System.Int32.Parse(dt_data[2]),
+						      System.Int32.Parse(dt_data[3]), 
+						      System.Int32.Parse(dt_data[4]), 
+						      System.Int32.Parse(dt_data[5]));
 			
 			return result;
 		}
-	
+		
 		public void SetData (byte [] data)
 		{
 			raw_data = data;
 			count = (uint)raw_data.Length / (uint)GetTypeSize ();
 		}
-		
+
+#if false		
+		public object  GetValue () {
+			switch (Type) {
+			case EntryType.Short:
+				return ShortValue;
+			case EntryType.Long:
+				return LongValue;
+			case  EntryType.Rational:
+				return RationalValue;
+			case EntryType.SRational:
+				return SRationalValue;
+			case EntryType.Ascii:
+				return StringValue.Split ('\0');
+				break;
+			default:
+				System.Console.WriteLine ("{1}({2}) [{0}]", this.Count, this.Id, this.Type);
+				break;
+
+				}
+			}
+		}
+#endif
+
 		public byte [] Value {
 			get {
 				return raw_data;
@@ -1055,12 +1413,49 @@ namespace FSpot.Tiff {
 			}
 		}
 
-		public string StringValue {
+		public string [] ValueAsString
+		{
 			get {
-				return System.Text.Encoding.ASCII.GetString (raw_data);
+				switch (this.Type) {
+				case EntryType.Short:
+				case EntryType.Long:
+					return ArrayToString (this.ValueAsLong);
+				case EntryType.Rational:
+					return ArrayToString (this.RationalValue);
+				case EntryType.SRational:
+					return ArrayToString (this.SRationalValue);
+				case EntryType.Undefined:
+					switch (Id) {
+					case TagId.UserComment:
+						return new string [] { UserCommentValue };
+					case TagId.FlashPixVersion:
+					case TagId.ExifVersion:
+						return new string [] { StringValue };
+					case TagId.FileSource:
+					case TagId.SceneType:
+						return ArrayToString (this.RawData);
+					default:
+						System.Console.WriteLine ("Cannot convert type \"{0}\" to string", Id);
+						break;
+					}
+					break;
+				case EntryType.Ascii:
+					return StringValue.Split ('\0');
+					break;
+				}
+				return null;
 			}
 		}
 
+		public string [] ArrayToString (System.Array array)
+		{
+			string [] vals = new string [array.Length];
+			for (int i = 0; i < array.Length; i++)
+				vals [i] = array.GetValue (i).ToString ();
+
+			return vals;
+		}
+
 		public uint [] ValueAsLong
 		{
 			get {
@@ -1073,6 +1468,9 @@ namespace FSpot.Tiff {
 					case EntryType.Short:
 						data [i] = BitConverter.ToUInt16 (raw_data, i * GetTypeSize (), endian == Endian.Little);
 						break;
+					case EntryType.Byte:
+						data [i] = raw_data [i];
+						break;
 					default:
 						throw new System.Exception ("Invalid conversion");
 					}
@@ -1080,14 +1478,67 @@ namespace FSpot.Tiff {
 				return data;
 			}
 		}
+		
+		// The following methods are usded to convert the data 
+		// to the various type regardless of the entry
+		// type, they are used internally in processing the data
+		// Use at your own risk.
+
+		public string StringValue {
+			get {
+				return System.Text.Encoding.ASCII.GetString (raw_data);
+			}
+		}
+
+		public System.DateTime ValueAsDate
+		{
+			get {
+				return DirectoryEntry.DateTimeFromString (StringValue);
+			}
+		}
+
+		public string UserCommentValue
+		{
+			get {
+				UserComment comment = new UserComment (raw_data, IsLittle);
+				return comment.Value;
+			}
+		}
+
+		public SRational [] SRationalValue
+		{
+			get {
+				Rational [] vals = RationalValue;
+				SRational [] data = new SRational [vals.Length];
+				
+				for (int i = 0; i < vals.Length; i++)
+					data [i] = SRational.BitwiseCopy (vals [i]);
+
+				return data;
+			}
+		}
+
+		public Rational [] RationalValue
+		{
+			get {
+				uint [] vals = LongValue;
+				Rational [] data = new Rational [vals.Length / 2];
+
+				for (int i = 0; i < vals.Length; i += 2)
+					data [i/2] = new Rational (vals [i], vals [i + 1]);
+				
+				return data;
+			}
+			
+		}
 
 		public uint [] LongValue
 		{
 			get {
-				uint [] data = new uint [raw_data.Length];
-				for (int i = 0; i < raw_data.Length; i+= 4) {
-					data [i] = BitConverter.ToUInt32 (raw_data, i, endian == Endian.Little);
-				}
+				uint [] data = new uint [raw_data.Length / 4];
+				for (int i = 0; i < raw_data.Length; i+= 4)
+					data [i/4] = BitConverter.ToUInt32 (raw_data, i, endian == Endian.Little);
+
 				return data;
 			}
 		}
@@ -1105,7 +1556,7 @@ namespace FSpot.Tiff {
 	}
 
 
-	public class TiffFile : ImageFile {
+	public class TiffFile : ImageFile, SemWeb.StatementSource {
 		public Header Header;
 
 		public TiffFile (string path) : base (path)
@@ -1117,6 +1568,11 @@ namespace FSpot.Tiff {
 				
 				ImageDirectory directory = Header.Directory;
 				while (directory != null) {
+#if TEST_METADATA 
+					FSpot.MetadataStore store = new FSpot.MetadataStore ();
+					Select (store);
+					//store.Dump ();
+#endif					
 					///directory.Dump ();
 					directory = directory.NextDirectory;
 				}
@@ -1127,14 +1583,21 @@ namespace FSpot.Tiff {
 			}
 		}
 
-		public override System.DateTime Date ()
+		public void Select (SemWeb.StatementSink sink)
 		{
-			AsciiEntry e = (AsciiEntry)(this.Header.Directory.Lookup (TagId.DateTime));
+			Header.Select (sink);
+		}
 
-			if (e != null)
-				return DirectoryEntry.DateTimeFromString (e.StringValue);
-			else
-				return base.Date ();
+		public override System.DateTime Date
+		{
+			get {
+				AsciiEntry e = (AsciiEntry)(this.Header.Directory.Lookup (TagId.DateTime));
+				
+				if (e != null)
+					return DirectoryEntry.DateTimeFromString (e.StringValue);
+				else
+					return base.Date;
+			}
 		}
 		
 		public override System.IO.Stream PixbufStream ()
@@ -1154,7 +1617,6 @@ namespace FSpot.Tiff {
 		public System.IO.Stream LookupJpegSubstream (ImageDirectory directory)
 		{
 			uint offset = directory.Lookup (TagId.JPEGInterchangeFormat).ValueAsLong [0];
-			//uint length = directory.Lookup (TagId.JPEGInterchangeFormatLength).ValueAsLong [0];
 			
 			System.IO.Stream file = System.IO.File.OpenRead (this.path);
 			file.Position = offset;
@@ -1278,15 +1740,17 @@ namespace FSpot.Tiff {
 		}
 
 
-		public override System.DateTime Date ()
+		public override System.DateTime Date
 		{
-			SubdirectoryEntry sub = (SubdirectoryEntry) this.Header.Directory.Lookup (TagId.ExifIfdPointer);
-			AsciiEntry e = (AsciiEntry)(sub.Directory [0].Lookup (TagId.DateTimeOriginal));
-
-			if (e != null)
-				return DirectoryEntry.DateTimeFromString (e.StringValue);
-			else
-				return base.Date ();
+			get {
+				SubdirectoryEntry sub = (SubdirectoryEntry) this.Header.Directory.Lookup (TagId.ExifIfdPointer);
+				AsciiEntry e = (AsciiEntry)(sub.Directory [0].Lookup (TagId.DateTimeOriginal));
+				
+				if (e != null)
+					return DirectoryEntry.DateTimeFromString (e.StringValue);
+				else
+					return base.Date;
+			}
 		}
 
 	}
